<?php

namespace Drupal\traits\Traits;

use Drupal\Core\Entity\EntityTypeManagerInterface;

/**
 * Trait TermTrait.
 */
trait TermTrait {

  /**
   * The terms serve as cache.
   *
   * @var array
   */
  protected $terms = [];

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Sets the entity type manager.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   */
  public function setMessenger(EntityTypeManagerInterface $entity_type_manager) {
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * Gets the entity type manager.
   *
   * @return \Drupal\Core\Entity\EntityTypeManagerInterface
   *   The entity type manager.
   */
  public function entityTypeManager() {
    if (!isset($this->entityTypeManager)) {
      $this->entityTypeManager = \Drupal::entityTypeManager();
    }
    return $this->entityTypeManager;
  }

  /**
   * Get tid by name.
   *
   * @param int $vid
   *   The vocabulary id.
   * @param string $name
   *   The name.
   *
   * @return int|false
   *   The tid of FALSE.
   */
  public function getExistingTid($vid, $name) {
    // Read from cache.
    if (isset($this->terms[$vid][$name])) {
      return $this->terms[$vid][$name];
    }

    $properties = [
      'name' => $name,
      'vid' => $vid,
    ];

    $storage = $this->entityTypeManager()->getStorage('taxonomy_term');
    $terms = $storage->loadByProperties($properties);
    if (!empty($terms)) {
      $term = reset($terms);
      $tid = $term->id();
      // Save the result to cache.
      $this->terms[$vid][$name] = $tid;
      return $tid;
    }
    return FALSE;
  }

  /**
   * Get tid by name if the term does not exist, create a new one.
   *
   * @param int $vid
   *   The vocabulary id.
   * @param string $name
   *   The name.
   *
   * @return int
   *   The tid.
   */
  public function getTid($vid, $name) {
    $tid = $this->getExistingTid($vid, $name);
    if ($tid === FALSE) {
      $storage = $this->entityTypeManager()->getStorage('taxonomy_term');
      $term = $storage->create(
        [
          'name' => $name,
          'vid' => $vid,
        ]);
      $term->save();
      $tid = $term->id();
      $this->terms[$vid][$name] = $tid;
    }
    return $tid;
  }

  /**
   * Add terms to vocabulary.
   *
   * @param string $vid
   *   The vid.
   * @param array $term_names
   *   The term names.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  public function addTerms($vid, array $term_names) {
    $storage = $this->entityTypeManager()->getStorage('taxonomy_term');
    if (!empty($term_names)) {
      foreach ($term_names as $term_name) {
        if (!$this->isTermExisted($vid, $term_name)) {
          $term_data = [
            'name' => $term_name,
            'vid' => $vid,
          ];
          $term = $storage->create($term_data);
          $term->save();
        }
      }
    }
  }

  /**
   * Checks that if the term exists.
   *
   * @param string $vid
   *   The vid.
   * @param string $name
   *   The term name to check.
   *
   * @return bool
   *   Exists or not.
   */
  public function isTermExisted($vid, $name) {
    $storage = $this->entityTypeManager()->getStorage('taxonomy_term');
    $query = $storage->getQuery();
    $query->condition('vid', $vid);
    $query->condition('name', $name);
    $result = $query->execute();
    if (!empty($result)) {
      return TRUE;
    }
    return FALSE;

  }

  /**
   * Create vocabulary or update the label if it exists and necessary.
   *
   * @param string $vid
   *   The vid.
   * @param string $vocabulary_label
   *   The vocabulary label.
   */
  private function createVocabulary($vid, $vocabulary_label) {

    $vocabulary_data = [
      'name' => $vocabulary_label,
      'vid' => $vid,
    ];

    $storage = $this->entityTypeManager()->getStorage('taxonomy_vocabulary');

    $vocabulary = $storage->load($vid);

    if ($vocabulary === NULL) {
      $vocabulary = $storage->create($vocabulary_data);
      $vocabulary->save();
    }
    elseif ($vocabulary->label() !== $vocabulary_label) {
      $vocabulary->name = $vocabulary_label;
      $vocabulary->save();
    }
  }

}
